Unit intermet;
{-----------------------------------------------------}
{                                                     }
{ This is my second component - InternetMeter.        }
{ Vladimir Kladov (C) 1997                            }
{                                                     }
{-----------------------------------------------------}

interface

uses
  Windows, SysUtils, Classes, DsgnIntf, Controls, ExtCtrls,
  Forms, Graphics, InterInfo;

const ArrSize = 4096;

type
  TNotifyConnection = procedure of object;

  TInterMeter = class(TCustomPanel)
  private
    FTestRead: Integer;
    FTestWrite: Integer;
    FTestMode: Boolean;
    procedure SetTestRead(const Value: Integer);
    procedure SetTestWrite(const Value: Integer);
    procedure SetTestMode(const Value: Boolean);
    { Private Declarations }
  protected
    { Protected Declarations }
    FDisplayWidth, FDisplayHeight : Integer;
    FRepaintAll : Boolean;
    FStable : Integer;
    FStartConnect : Boolean;
    FPrevLong : String;
    FDisplay : TBitMap;
    FBevelOuter : TPanelBevel;
    FDisplayColor : TColor;
    FInColor : TColor;
    FOutColor : TColor;
    FTimer : TTimer;
    FLastReceived, FLastSent, FSpeed : Integer;
    FOnConnect : TNotifyConnection;
    FOnDisconnect : TNotifyConnection;
    FOnStable : TNotifyConnection;
    FOnScan : TNotifyConnection;
    FTimeout : Integer;
    FMaxSpeed : Integer;
    FArray : array[1..ArrSize] of Integer;
    FPos : Integer;
    IsConnected : Boolean;
    FUseSent : Boolean;
    FDesign : Boolean;
    FCounter : Integer;
    FInternetInfo : TInternetInfo;
    procedure Paint; override;
    procedure TimerTick( Sender : TObject );
    procedure SetTimeout( T : Integer );
    function IsActive : Boolean;
    procedure Activate( E : Boolean );
    procedure SetInColor( C : TColor );
    procedure SetOutColor( C : TColor );
    procedure SetDisplayColor( C : TColor );
    procedure SetBevelOuter( B : TPanelBevel );
    procedure SetDisplayWidth( W : Integer );
    procedure SetDisplayHeight( H : Integer );
  public
    { Public Declarations }
    FMemory : PChar;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property TestMode : Boolean read FTestMode write SetTestMode;
    property TestRead : Integer read FTestRead write SetTestRead;
    property TestWrite : Integer read FTestWrite write SetTestWrite;
    property Received: Integer read FLastReceived;
    property Sent: Integer read FLastSent;
    property Speed: Integer read FSpeed;
    property Connected: Boolean read IsConnected;
  published
    property OnConnect : TNotifyConnection read FOnConnect
                                           write FOnConnect;
    property OnDisconnect : TNotifyConnection read FOnDisconnect
                                              write FOnDisconnect;
    property OnStable : TNotifyConnection read FOnStable
                                          write FOnStable;
    property OnScan : TNotifyConnection read FOnScan
                                        write FOnScan;
    property TimeOut : Integer read FTimeout
                               write SetTimeout
                               default 5;
    property BevelOuter : TPanelBevel read FBevelOuter
                                      write SetBevelOuter
                                      default bvLowered;
    property DisplayColor : TColor read FDisplayColor
                                   write SetDisplayColor
                                   default clBtnFace;
    property InColor : TColor read FInColor
                              write SetInColor
                              default clGreen;
    property OutColor : TColor read FOutColor
                               write SetOutColor
                               default clRed;
    property UseSentToDetermineStable : Boolean
                                      read FUseSent
                                      write FUseSent
                                      default False;
    property Active : Boolean read IsActive
                               write Activate
                               default True;
    property DesignActive : Boolean read FDesign write FDesign
                          default False;
    property DisplayWidth : Integer read FDisplayWidth write SetDisplayWidth
                                    default 1024;
    property DisplayHeight : Integer read FDisplayHeight write SetDisplayHeight
                                     default 40;
    property Align;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnKeyDown;
    property OnKeyPress;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Internet', [TInterMeter]);
end;

{--------------------------------------}
{ TInterMeter.Create                   }
{--------------------------------------}
constructor TInterMeter.Create(AOwner: TComponent);
var I : Integer;
begin
  inherited Create(AOwner);
  Width := 56;
  Height := 56;
  DisplayWidth := 1024;
  DisplayHeight := 40;
  Visible := True;
  Enabled := True;
  FBevelOuter := bvLowered;
  FDisplayColor := Color;
  FInColor := clGreen;
  FOutColor := clRed;
  for I := 1 to ArrSize do
  begin
      FArray[I] := 0;
      //FArray[I] := Random(1000)*10000 + Random(1000);
  end;
  FPos := 1;
  FMaxSpeed := 14400 div 8;
  FTimer := TTimer.Create( Self );
  FTimer.Interval := 1000;
  FTimer.OnTimer := TimerTick;
  FTimer.Enabled := (csDesigning in ComponentState)
                    and FDesign
                    or not (csDesigning in ComponentState);
  FDisplay := TBitmap.Create;
  FDisplay.Width := 1;
  FDisplay.Height := 1;
  FTimeout := 5;
  FInternetInfo := TInternetInfo.Create;
end;

{--------------------------------------}
{ TInterMeter.Destroy                  }
{--------------------------------------}
destructor TInterMeter.Destroy;
begin
  FDisplay.Destroy;
  FInternetInfo.Destroy;
  inherited;
end;

procedure TInterMeter.SetTimeout( T : Integer );
begin
  FStable := 0;
  FTimeout := T;
end;

procedure TInterMeter.Paint;
var DRect : TRect;
    I, J, K, L : Integer;
    X1, X2 : Integer;
begin
  DRect := ClientRect;
  with Drect, Canvas do
  for I := 1 to 3 do
  begin
    MoveTo(Right,Top);
    if FBevelOuter <> bvNone then
    begin
       Pen.Color := clBtnFace;
       if FBevelOuter = bvLowered then Pen.Color := clBtnShadow;
       if FBevelOuter = bvRaised then Pen.Color := clBtnHighLight;
       LineTo(Left,Top);
       LineTo(Left,Bottom);
       if FBevelOuter = bvLowered then Pen.Color := clBtnHighLight;
       if FBevelOuter = bvRaised then Pen.Color := clBtnShadow;
       LineTo(Right,Bottom);
       LineTo(Right,Top);
       Inc(Left); Dec(Right);
       Inc(Top); Dec(Bottom);
    end;
  end;
  J := FPos;
  if (FDisplay.Width <> FDisplayWidth ) or
     (FDisplay.Height <> FDisplayHeight )
     or FRepaintAll then
  begin
    FDisplay.Width := FDisplayWidth;
    FDisplay.Height := FDisplayHeight;
    X2 := FDisplay.Width - 1;
    X1 := 0;
    FRepaintAll := False;
  end
  else
  begin
    X2 := FDisplay.Width - 1;
    X1 := X2;
  end;
  with FDisplay.Canvas, FDisplay do
  for I := X2 downto X1 do
  begin
    K := FArray[J];
    if K = 0 then K := 1;
    K := K mod 10000;
    if FMaxSpeed = 0 then FMaxSpeed := 1;
    K := Height - K * Height div FMaxSpeed;
    if K < 0 then K := 0;
    MoveTo(I,Height + 1);
    Pen.Color := FOutColor;
    LineTo(I, K );
    L := FArray[J] div 10000;
    L := K - L * Height div FMaxSpeed;
    if L < 0 then L := 0;
    Pen.Color := FInColor;
    LineTo(I, L );
    Pen.Color := FDisplayColor;
    LineTo(I, -1);
    Dec(J);
    if J=0 then J := ArrSize;
  end;
  Canvas.CopyRect( Rect( DRect.Left, DRect.Top, DRect.Right, DRect.Bottom ),
                   FDisplay.Canvas,
                   Rect( FDisplay.Width - (DRect.Right-DRect.Left),
                         0, FDisplay.Width, FDisplay.Height ) );
end;

procedure TInterMeter.TimerTick( Sender : TObject );
var Speed, Received, Sent : Integer;
    Connect, RealConnect, Disconnect, PossibStable, Stable : Boolean;
begin
  if (csDesigning in ComponentState) then
  if not DesignActive then Exit;
  Connect := False;
  RealConnect := False;
  Disconnect := False;
  Stable := False;

  if not TestMode then
  begin

     if FInternetInfo.InternetWindow <> 0 then // widow was found already
        if not IsWindow( FInternetInfo.InternetWindow ) then // Disconnecting
           Disconnect := True;

     if FInternetInfo.InternetWindow = 0 then // attempt to find window
     begin
        FInternetInfo.SearchConnectWin;
        if FInternetInfo.InternetWindow <> 0 then
           Connect := True;
     end;

  end
     else
     if not Connected then
     begin
        Connect := True;
        IsConnected := True;
     end;

  Speed := 0; Received := 0; Sent := 0;
  if (FInternetInfo.InternetWindow <> 0) or TestMode then
  begin
    if not FInternetInfo.GetConnectInfo( Speed, Received, Sent ) then
       Disconnect := True;
    if FTestRead <> 0 then
    begin
       Inc( Received, FTestRead );
    end;
    if FTestWrite <> 0 then
    begin
       Inc( Sent, FTestWrite );
    end;
    PossibStable := (Received = FLastReceived);
    if FUseSent then PossibStable := PossibStable and (Sent = FLastSent);
    if PossibStable and not FStartConnect then
    begin
      Inc( FStable );
      if FStable = FTimeout then
         Stable := True;
    end
    else FStable := 0;
    if (Speed=0) and (Received=0) and (Sent=0) and Stable then
    begin
      Disconnect := True;
      Connect := False;
    end;
  end;

  Inc( FPos );
  if FPos >= ArrSize then FPos := 1;
  if Connect then
  begin
     FLastReceived := Received;
     FLastSent := Sent;
  end;
  if Received = 0 then
  if Sent = 0 then
  begin
    Received := FLastReceived;
    Sent := FLastSent;
  end;
  FArray[FPos] := (Received - FLastReceived)*10000 + (Sent-FLastSent);
  FDisplay.Canvas.CopyRect(
              Rect( 0, 0, FDisplay.Width - 1, FDisplay.Height + 1 ),
              FDisplay.Canvas,
              Rect( 1, 0, FDisplay.Width, FDisplay.Height + 1 ) );
  Repaint;
  if FStartConnect then
  if FLastReceived <> Received then
  begin
     FStartConnect := False;
     RealConnect := True;
  end;
  FLastReceived := Received;
  FLastSent := Sent;

  if FSpeed = 0 then
  begin
    FSpeed := Speed;
    if Speed <> 0 then
       FMaxSpeed := Speed div 8;
  end;

  if Disconnect then
  begin
    IsConnected := False;
    if Assigned( FOnDisconnect ) then
       FOnDisconnect;
  end
  else
  if Connect then
  begin
    FStartConnect := True;
  end
  else
  if Stable then
  begin
    if Assigned( FOnStable ) then
       FOnStable;
  end
  else
  if RealConnect then
  begin
    IsConnected := True;
    if Assigned( FOnConnect ) then
       FOnConnect;
  end;

  if Assigned( FOnScan ) then
     FOnScan;
end;

function TInterMeter.IsActive : Boolean;
begin
  Result := False;
  if FTimer <> nil then
  Result := FTimer.Enabled;
end;

procedure TInterMeter.Activate( E : Boolean );
begin
  if (csDesigning in ComponentState) and
     not FDesign then E := False;
  if FTimer <> nil then
  FTimer.Enabled := E;
end;

procedure TInterMeter.SetInColor( C : TColor );
begin
  FInColor := C;
  FRepaintAll := True;
  RePaint;
end;

procedure TInterMeter.SetOutColor( C : TColor );
begin
  FOutColor := C;
  FRepaintAll := True;
  Repaint;
end;

procedure TInterMeter.SetDisplayColor( C : TColor );
begin
  FDisplayColor := C;
  Color := C;
  FRepaintAll := True;
  Repaint;
end;

procedure TInterMeter.SetBevelOuter( B : TPanelBevel );
begin
  FBevelOuter := B;
  FRepaintAll := True;
  Repaint;
end;

procedure TInterMeter.SetDisplayWidth( W : Integer );
begin
  if W = 0 then W := 1024;
  if W < 100 then W := 100;
  if W > ArrSize then W := ArrSize;
  FDisplayWidth := W;
end;

procedure TInterMeter.SetDisplayHeight( H : Integer );
begin
  if H = 0 then H := 40;
  if H < 10 then H := 10;
  if H > 64 then H := 64;
  FDisplayHeight := H;
end;

procedure TInterMeter.SetTestRead(const Value: Integer);
begin
  FTestRead := Value;
  FSpeed := 14400;
  FTestMode := True;
end;

procedure TInterMeter.SetTestWrite(const Value: Integer);
begin
  FTestWrite := Value;
  FSpeed := 14400;
  FTestMode := True;
end;

procedure TInterMeter.SetTestMode(const Value: Boolean);
begin
  FTestMode := Value;
end;

end.

